import mercadopago
from iso8601 import parse_date

from ..response import ok, error


def _format_card(response):
    return {
        'id': response['card']['id'],
        'card_type': response['payment_type_id'],
        'last_four_digits': response['card']['last_four_digits'],
        'cardholder': {
            'name': response['card']['cardholder']['name']
        },
        'expiration_month': response['card']['expiration_month'],
        'expiration_year': response['card']['expiration_year'],
    }


def _format(response):
    return {
        'id': response['id'],
        'transaction_amount': response['transaction_amount'],
        'date_created': parse_date(response['date_created']),
        'status': response['status'],
        'status_detail': response['status_detail'],
        'card': _format_card(response),
        'processor': 'mercadopago'
    }


class Payment:
    def __init__(self, sdk: mercadopago.SDK):
        self.sdk = sdk

    def create(self, payment_data: dict):
        payment_data = payment_data.copy()
        payment_data_mp = {
            "notification_url": None,
            "transaction_amount": round(payment_data['transaction_amount'], 2),
            "token": payment_data['token'],
            "installments": 1,
        }

        optional_fields = [
            'additional_info',
            'description',
            'external_reference',
            'point_of_interaction',
            'installments',
            'payment_method_id',
            'application_fee'
        ]
        for f in optional_fields:
            if f in payment_data:
                payment_data_mp[f] = payment_data[f]

        if 'payer' in payment_data:
            payment_data_mp["payer"] = {
                "first_name": payment_data['payer']["nombre"],
                "last_name": payment_data['payer']["apellido"],
                "address": None
            }
            if 'mercadopago_id' in payment_data['payer']:
                payment_data_mp['payer']['id'] = payment_data['payer']["mercadopago_id"]
            elif 'email' in payment_data['payer']:
                payment_data_mp['payer']['email'] = payment_data['payer']["email"]

        payment_response = self.sdk.payment().create(payment_data_mp)
        if payment_response['status'] < 400:
            return ok(_format(payment_response['response']))

        payment_response['response']['processor'] = 'mercadopago'
        return error(payment_response['response'])

    def get(self, payment_id: int):
        payment_response = self.sdk.payment().get(payment_id)
        if payment_response['status'] < 400:
            return ok(payment_response['response'])

        return error(payment_response['response'])



import mercadopago

from ..response import ok, error


def _format(response):
    return {
        'id': response['id'],
        'card_type': response['payment_method']['payment_type_id'],
        'card_brand': response["payment_method"]["name"],
        'last_four_digits': response['last_four_digits'],
        'issuer': {
            'name': response['issuer']['name']
        },
        'cardholder': {
            'name': response['cardholder']['name']
        },
        'expiration_month': response['expiration_month'],
        'expiration_year': response['expiration_year'],
        'payer_id': response['user_id']
    }


class Card:
    def __init__(self, sdk: mercadopago.SDK):
        self.sdk = sdk

    def create(self, customer_id: str, card_token: str):
        card_data = {'token': card_token}
        mp_response = self.sdk.card().create(customer_id, card_data)

        if mp_response['status'] < 400:
            return ok(_format(mp_response['response']))

        return error(mp_response['response']['error'])
        
    def get(self, customer_id: str, card_id: str) -> dict:
        mp_response = self.sdk.card().get(customer_id, card_id)
        if mp_response['status'] < 400:
            card = mp_response['response']
            return ok(_format(card))

        return error(mp_response['response']['message'])

    def list_all(self, customer_id: str):
        cards = self.sdk.card().list_all(customer_id)
        if cards['status'] > 299:
            return error('Customer not found')
        else:
            cards = cards['response']
            return ok([_format(card) for card in cards])



from .mercadopago_sdk.MercadoPagoSDK import MercadoPagoSDK
from .decidir_sdk.DecidirSDK import DecidirSDK


class SDK:
    def __init__(self, merchant: dict, processor: str = None, merchant_cadena: dict = None):
        # Si se le indica el procesador
        self.processor = processor
        if processor == 'mercadopago' and merchant['credentials']['mercadopago']['active']:
            self._sdk = MercadoPagoSDK(merchant)
            return
        elif processor == 'mp_marketplace' and merchant['credentials']['mp_marketplace']['active']:
            self._sdk = MercadoPagoSDK(merchant, marketplace=True)
            return
        elif processor == 'decidir' and merchant['credentials']['decidir']['active']:
            self._sdk = DecidirSDK(merchant, merchant_cadena)

    def customer(self):
        return self._sdk.customer()

    def card(self):
        return self._sdk.card()

    def card_token(self):
        return self._sdk.card_token()

    def payment(self):
        return self._sdk.payment()

    def refund(self):
        return self._sdk.refund()

    def which(self):
        return self.processor




import requests
import json
import random
import string
from iso8601 import parse_date
from ..response import ok, error

def _format_card(response):
    if 'card' not in response:
        return {
            'id': '',
            'card_type': '',
            'last_four_digits': '',
            'cardholder': {
                'name': ''
            },
            'expiration_month': '',
            'expiration_year': '',
        }
    else:
        return {
            'id': response['card']['id'],
            'card_type': 'not_found',
            'last_four_digits': response['card']['last_four_digits'],
            'cardholder': {
                'name': response['card']['cardholder']['name']
            },
            'expiration_month': response['card']['expiration_month'],
            'expiration_year': response['card']['expiration_year'],
        }

def _format(response):
    if response['status'] == 'accredited':
        response['status'] = 'approved'
        response['status_detail'] = 'accredited'
    elif response['status'] == 'approved':
        response['status_detail'] = 'not_accredited'
    elif response['status'] == 'rejected':
        response['status_detail'] = 'rejected'
    result = {
        'id': response['site_transaction_id'],
        'transaction_amount': response['amount']/100,
        'date_created': parse_date(response['date']),
        'status': response['status'],
        'status_detail': response['status_detail'],
        'card': _format_card(response),
        'processor': 'decidir'
    }
    return result

class Payment:
    def __init__(self, url, private_key: str, public_key: str, payment_type: str, site_id: str, seller_site_id: str):
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.payment_type = payment_type
        self.site_id = site_id
        self.seller_site_id = seller_site_id
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, payment_data: dict):
        site_transaction_id = ''.join(random.choices(string.ascii_letters + string.digits, k=10))
        total_amount = round(payment_data['transaction_amount'], 2) * 100  # se multiplica por 100; no acepta decimales
        body = {
            'customer': {
                'id': payment_data['payer']['decidir_id'],
                'email': payment_data['payer']['email']
            },
            'site_transaction_id': site_transaction_id,     # str(uuid4())
            'token': payment_data['token'],
            'payment_method_id': 1,    # Peso Argentino
            'bin': payment_data['bin'],    # primeros 6 digitos de la tarjeta
            'currency': 'ARS',
            'installments': 1,
            'payment_type': self.payment_type,
            'establishment_name': '-',  # opcional
            'sub_payments': []
        }
        if self.payment_type == 'distributed':
            application_fee = round(payment_data['application_fee'], 2) * 100
            body['sub_payments'] = [
                {
                    'site_id': payment_data['seller_site_id'],
                    'installments': 1,
                    'amount': total_amount - application_fee
                }, {
                    'site_id': self.site_id,    # Cadena
                    'installments': 1,
                    'amount': application_fee
                }
            ]
        else:
            body['amount'] = total_amount
            body['site_id'] = self.site_id

        additional_information = [
            'installments',
            'establishment_name'
        ]
        for item in additional_information:
            if item in payment_data.keys():
                body[item] = payment_data[item]

        r = requests.post(self.url + '/payments', headers=self.headers, data=json.dumps(body))
        response = r.json()
        if r.status_code < 400:
            if 'card' in payment_data:
                response['card'] = payment_data['card']
            return ok(_format(response))
        return error(response)

    def get(self, payment_id: str):
        # r = requests.get(self.url + '/payments/{}'.format(payment_id), headers=self.headers)
        # return ok(_format(r.json()))
        filters = {'siteOperationId': payment_id,
                   'expand':'card_data'}
        r = requests.get(self.url + '/payments', params=filters, headers=self.headers)
        list_results = r.json()['response']['results']
        if len(list_results) == 1:
            return _format(list_results[0])
        else:
            return error('error_pago_no_encontrado')

    def search(self, filters: dict) -> dict:
        # El filter puede ser:
        # offset
        # pageSize
        # siteOperationId
        # merchantId
        # dateFrom
        # dateTo
        # site
        r = requests.get(self.url + '/payments', params=filters, headers=self.headers)
        response = r.json()
        response['results'] = [_format(r) for r in response['results']]
        return ok(response)

    def get_payment_methods(self):
        r = requests.get(self.url + '/payment-methods/1', headers=self.headers)
        return r.json()



from ..response import ok, error


def _format(response: dict) -> dict:
    return {
        'email': response['email'],
        'first_name': response['first_name'],
        'last_name': response['last_name'],
        'id': response['id']
    }


def validate_client(client: dict) -> dict:
    fields = ['nombre', 'apellido', 'documento', 'email', 'domicilio']
    for f in fields:
        if f not in client:
            return error('{} field is missing'.format(f))

    for f in ['calle', 'altura']:
        if f not in client['domicilio']:
            return error('domicilio.{} field is missing'.format(f))

    return ok(client)


class Customer:
    def __init__(self, sdk):
        self.sdk = sdk

    def create(self, client: dict) -> dict:
        """

        :param client: Diccionario del cliente
        nombre, apellido, documento, email, domicilio son obligatorios
        :return:
        """
        # Check para el client y evitar errores
        r = validate_client(client)
        if r['status'] == 'error':
            return r

        customer_response = self.sdk.customer().search(filters={'email': client['email']})
        if len(customer_response['response']['results']) > 0:
            return ok(_format(customer_response['response']['results'][0]))

        try:
            street_number = int(client['domicilio']['altura'])
        except:
            street_number = 1

        mp_customer_data = {
            'email': client['email'],
            'first_name': client['nombre'],
            'last_name': client['apellido'],
            'phone': {
                'area_code': None,
                'number': client['celular']
            },
            'identification': {
                'type': 'DNI',
                'number': client['documento']
            },  # ! Tipo de identificacion hardcodeado
            'address': {
                'street_name': client['domicilio']['calle'],
                'street_number': street_number,
            }
        }
        mp_response = self.sdk.customer().create(mp_customer_data)
        response_status = mp_response['status']
        if response_status < 400:
            return ok(_format(mp_response['response']))

        return error(mp_response['response']['message'])

    def get(self, customer_id: str):
        mp_response = self.sdk.customer().get(customer_id)
        if mp_response['status'] < 400:
            customer = mp_response['response']
            return ok(_format(customer))

        return error(mp_response['response']['message'])

    def search(self, filters: dict):
        mp_response = self.sdk.customer().search(filters)
        if mp_response['status'] < 400:
            mp_response['status'] = 'ok'
            return mp_response

        return error(mp_response['response']['message'])



from .Customer import Customer
from .Card import Card
from .CardToken import CardToken
from .Payment import Payment
from .Refund import Refund


class DecidirSDK:
    def __init__(self, merchant, merchant_cadena=None,site_id=None):
        if merchant_cadena:
            credentials = merchant_cadena['credentials']['decidir']
            self.payment_type = 'distributed'
        else:
            credentials = merchant['credentials']['decidir']
            self.payment_type = 'single'
        self.url = 'https://developers.decidir.com/api/v2'
        self.private_key = credentials['access_token']
        self.public_key = credentials['public_key']
        self.merchant_id = merchant['_id']
        self.merchant_name = merchant['name']
        self.site_id = site_id

    def customer(self):
        return Customer(self.url, self.private_key, self.public_key)
    
    def card(self):
        return Card(self.url, self.private_key, self.public_key)

    def card_token(self):
        return CardToken(self.url, self.private_key, self.public_key)

    def payment(self):
        return Payment(self.url, self.private_key, self.public_key, self.payment_type)

    def refund(self):
        return Refund(self.url, self.private_key, self.public_key)



import mercadopago
from .Customer import Customer
from .Card import Card
from .CardToken import CardToken
from .Payment import Payment
from .Refund import Refund


class MercadoPagoSDK:
    def __init__(self, merchant, marketplace=False):
        if 'credentials' in merchant:
            if not marketplace:
                access_token = merchant['credentials']['mercadopago']['access_token']
            else:
                access_token = merchant['credentials']['mp_marketplace']['access_token']
        else:
            if not marketplace:
                access_token = merchant['keys']['access_token']
            else:
                access_token = merchant['keys_marketplace']['access_token']
        self.sdk = mercadopago.SDK(access_token)
        self.merchant_id = merchant['_id']
        self.merchant_name = merchant['name']

    def customer(self):
        return Customer(self.sdk)
    
    def card(self):
        return Card(self.sdk)

    def card_token(self):
        return CardToken(self.sdk)
    
    def payment(self):
        return Payment(self.sdk)

    def refund(self):
        return Refund(self.sdk)



import requests

from ..response import ok

card_brand = {
    1: 'Visa',
    8: 'Diners Club',
    23: 'Tarjeta Shopping',
    24: 'Tarjeta Naranja',
    25: 'PagoFacil',
    26: 'RapiPago',
    29: 'Italcred',
    30: 'ArgenCard',
    34: 'CoopePlus',
    37: 'Nexo',
    38: 'Credimás',
    39: 'Tarjeta Nevada',
    42: 'Nativa',
    43: 'Tarjeta Cencosud',
    44: 'Tarjeta Carrefour / Cetelem',
    45: 'Tarjeta PymeNacion',
    48: 'Caja de Pagos',
    50: 'BBPS',
    51: 'Cobro Express',
    52: 'Qida',
    54: 'Grupar',
    55: 'Patagonia 365',
    56: 'Tarjeta Club Día',
    59: 'Tuya',
    60: 'Distribution',
    61: 'Tarjeta La Anónima',
    62: 'CrediGuia',
    63: 'Cabal Prisma',
    64: 'Tarjeta SOL',
    65: 'American Express',
    103: 'Favacard',
    104: 'MasterCard Prisma',
    109: 'Nativa Prisma',
    111: 'American Express Prisma',

    # Debito
    31: 'Visa Débito',
    105: 'MasterCard Debit Prisma',
    106: 'Maestro Prisma',
    108: 'Cabal Débito Prisma'
}


def _card_type(payment_method_id):
    if payment_method_id in [31, 105, 106, 108]:
        return 'credit_card'
    return 'debit_card'


def _format(response, customer_id):
    return {
        'id': response['token'],
        'card_type': _card_type(response['payment_method_id']),
        'card_brand': card_brand[response['payment_method_id']],
        'last_four_digits': response['last_four_digits'],
        'issuer': {
            'name': card_brand[response['payment_method_id']]
        },
        'cardholder': {
            'name': response['card_holder']['name']
        },
        'expiration_month': response['expiration_month'],
        'expiration_year': response['expiration_year'],
        'payer_id': customer_id
    }


class Card:
    def __init__(self, url: str, private_key: str, public_key: str):
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, customer_id: str, card_token: str):
        """ Tokenizar la tarjeta

        :param customer_id: Id del cliente
        :type customer_id: str
        :param card_token: Token de la tarjeta
        :type card_token: str
        :return: Tarjeta con su token
        :rtype: dict
        """
        response = {
                'id': card_token
            }
        return ok(response)

    def get(self, customer_id: str, card_id: str):
        r = requests.get('{}/usersite/{}/cardtokens'.format(self.url, customer_id), headers=self.headers)
        cards = r.json()['tokens']

        for card in cards:
            if card['token'] == card_id:
                return ok(_format(card, customer_id))

    def list_all(self, customer_id: str):
        r = requests.get('{}/usersite/{}/cardtokens'.format(self.url, customer_id), headers=self.headers)
        cards = r.json()['tokens']
        return ok([_format(card, customer_id) for card in cards])



import mercadopago

from ..response import ok, error


class CardToken:
    def __init__(self, sdk: mercadopago.SDK):
        self.sdk = sdk

    def create(self, card: dict):
        if 'mercadopago_id' in card:
            card['id'] = card['mercadopago_id']

        data = {
            "card_id": card['id']
        }
        card_token_response = self.sdk.card_token().create(data)

        if card_token_response["status"] > 299:
            return error("Falló la creación de token de la tarjeta.")
        return ok(card_token_response["response"])



import requests
import json

class Refund:
    def __init__(self, url: str, private_key: str, public_key: str):
        self.url = url
        self.private_key = private_key
        self.public_key = public_key
        self.headers = {
            'apikey': self.private_key,
            'Content-Type': 'application/json',
            'Cache-Control': 'no-cache'
        }

    def create(self, payment_id: str, amount: float=None) -> dict:
        data = {
            'amount': round(amount,2)*100 if amount else amount
        }
        r = requests.post(self.url + f'/payments/{payment_id}/refunds', headers=self.headers, data=json.dumps(data))
        return r.json()




import mercadopago
from iso8601 import parse_date
from ..response import ok, error

class Refund:
    def __init__(self, sdk: mercadopago.SDK):
        self.sdk = sdk

    def create(self, payment_id:str,amount:float=None):
        refund_data = {
            "amount": amount
        }
        refund_response = self.sdk.refund().create(payment_id,refund_data)
        if refund_response['status'] < 400:
            return ok(refund_response['response'])
        else:
            return error(refund_response['response'])



import setuptools
import subprocess

remote_version = (
    subprocess.run(["git", "describe", "--tags"], stdout=subprocess.PIPE)
    .stdout.decode("utf-8")
    .strip()
)
assert "." in remote_version

setuptools.setup(
    name="muvisdk",
    version=remote_version,
    author="muvinai",
    description="SDK for muvinai developers",
    long_description="SDK Muvinai Python package",
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(where="src"),
    package_data={"muvisdk": ["VERSION"]},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    python_requires=">=3.6",
    install_requires=[
        'mercadopago'
    ],
)
